"""
@Description: 
@Usage: 
@Author: liuxianglong
@Date: 2021/8/21 下午6:44
"""
# !/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='scrapy_box',
    version='V1.4.0',
    description=(
        '简单易用的scrapy插件盒'
    ),
    long_description=open('README.rst').read(),
    author='xlomg',
    author_email='liu_xianglong@live.com',
    maintainer='xlomg',
    maintainer_email='liu_xianglong@live.com',
    license='BSD License',
    packages=['scrapy_box.middlewares', 'scrapy_box.pipelines'],
    py_modules=["scrapy_box.__init__"],
    platforms=["all"],
    url='https://github.com/xlomg/scrapy_box.git',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'scrapy', 'treq', 'pymongo', 'pandas', 'pymysql'
    ]
)